/* Polytropic equation of state -- relativstic approximation */

#include <stdio.h>
#include <math.h>
#include <stdlib.h>
#ifndef PI
#define PI 4.0*atan(1.0)
#endif

/* Some constants: */
#define C          3e8
#define G          6.67e-11
#define HBAR       1.055e-34
#define M_SOLAR    1.99e30
#define R_SOLAR    6.96e8
#define M_N_MKS    1.67e-27
#define DR         0.0001
#define MAXSTEPS   1000000

/* ============================================== */
double DmDr (double r, double rho)
{
  return r*r*rho;
}
/* ============================================== */
double DrhoDr (double r, double rho, double m) 
{
  if (rho < 0.0) 
    return 0.0;
  else
    return -m*pow(rho, 2.0/3.0)/(r*r);
}
/* ============================================== */
int main (int argc, char *argv[])
{
  double rho, r, m;
  double R0, M0, RHO0;
  double alpha, rho_c;
  double k1m,k2m,k3m,k4m,k1rho,k2rho,k3rho,k4rho;
  FILE *fp;

  if (argc != 3) {
    printf("Wrong arguments, exiting...\n");
    exit(2);
  }
  alpha = atof(argv[1]);
  rho_c = atof(argv[2]);

  RHO0 = (9.79e8)/alpha;
  R0 = sqrt(pow(3.0*PI*PI,1.0/3.0)*HBAR*C/(12.0*PI*G*pow(RHO0,2.0/3.0))
	    *pow(alpha/M_N_MKS,4.0/3.0));
  M0 = 4.0*PI*pow(R0,3.0)*RHO0;

  printf("\nalpha =\t%le\n", alpha);
  printf("rhoc =\t%le\n", rho_c);
  printf("Computed:\n");
  printf("RHO0 =\t%le\n", RHO0);
  printf("R0   =\t%le\n", R0);
  printf("M0   =\t%le\n", M0);
  printf("\n");

  /* (Approximate) starting conditions: */
  r   = DR;
  rho = rho_c;
  m   = (1.0/3.0)*pow(r,3)*rho_c;

  fp = fopen("density", "w");

  while (r < MAXSTEPS*DR) {

    fprintf(fp, "%le %le\n", r, rho);

    /* Take a RK step: */
    k1m = DR*DmDr(r, rho);
    k1rho = DR*DrhoDr(r, rho, m);

    k2m = DR*DmDr(r + DR/2.0, rho + k1rho/2.0);
    k2rho = DR*DrhoDr(r + DR/2.0, rho + k1rho/2.0, m + k1m/2.0);

    k3m = DR*DmDr(r + DR/2.0, rho + k2rho/2.0);
    k3rho = DR*DrhoDr(r + DR/2.0, rho + k2rho/2.0, m + k2m/2.0);

    k4m = DR*DmDr(r + DR, rho + k3rho);
    k4rho = DR*DrhoDr(r + DR, rho + k3rho, m + k3m);	      

    m   += (k1m + k4m)/6.0 + (k2m + k3m)/3.0;
    rho += (k1rho + k4rho)/6.0 + (k2rho + k3rho)/3.0;
    r   += DR;

    /* Check if we have reached the edge of the star: */
    if (rho <= 0.0) {
      printf("\n");
      printf("Radius (km):\t%4.3e\n", r*R0/1000.0);
      printf("Mass (M_sun):\t%4.3e\n", m*M0/M_SOLAR);
      fclose (fp);
      exit(0);
    }
  }
  /* If we get here, the edge of the star was not reached... */
  printf("Model failed to converge after %d steps\n", MAXSTEPS);
  printf("Final r   = %le\n", r);
  printf("Final rho = %le\n", rho);
  printf("Final m   = %le\n", m);
  fclose (fp);
  return 1;
}

