/* Solves the equilibrium equations for a white dwarf with the full
   degenerate electron gas equation of state and 4th order RK
   integration.

   Command line arguments are:
   * alpha (ratio of protons to nucleons in the star)
   * rhobar_c (dimensionless central density)

   The density profile is written to a file "density", and the final
   mass and radius are printed to stdout. */

#include <stdio.h>
#include <math.h>
#include <stdlib.h>

#ifndef PI
#define PI 4.0*atan(1.0)
#endif

/* Some constants (MKS units): */
#define C          3e8
#define G          6.67e-11
#define M_SOLAR    1.99e30
#define R_SOLAR    6.96e8
#define M_ELECTRON 9.11e-31
#define M_NUCLEON  1.674e-27
#define DR         0.0001
#define MAXSTEPS   100000

/* For the degenerate neutron gas, replace M_E -> M_N above and
   multiply RHO0 by (m_N/M_E)^3 */

/* ============================================== */
double gam (double x) 
{
  if (x < 0.0) 
    return 0.0;
  else
    return x*x/(3.0*sqrt(1.0 + x*x));
}
/* ============================================== */
double DmDr (double r, double rho) 
{
  return r*r*rho;
}
/* ============================================== */
double DrhoDr (double r, double rho, double m) 
{
  double x;
  if (rho < 0.0) return 0.0;
  x = pow(rho, 0.333333);
  return -m*rho/(r*r*gam(x));
}
/* ============================================== */
int main (int argc, char *argv[])
{
  double rho, r, m;
  double R0, M0, RHO0;
  double alpha, rho_c;
  double k1m,k2m,k3m,k4m,k1rho,k2rho,k3rho,k4rho;
  FILE *fp;

  if (argc != 3) {
    printf("Wrong arguments, exiting...\n");
    exit(2);
  }
  alpha = atof(argv[1]);
  rho_c = atof(argv[2]);

  RHO0 = (9.79e8)/alpha;
  R0 = C*sqrt(alpha*M_ELECTRON/(M_NUCLEON*4.0*PI*G*RHO0));
  M0 = 4.0*PI*pow(R0,3)*RHO0;

  printf("\nalpha =\t%le\n", alpha);
  printf("rhoc  =\t%le\n", rho_c);
  printf("Computed:\n");
  printf("RHO0 =\t%le\n", RHO0);
  printf("R0   =\t%le\n", R0);
  printf("M0   =\t%le\n", M0);
  printf("\n");

  /* Starting conditions: */
  r   = DR;
  rho = rho_c - rho_c*rho_c*DR*DR/6.0;
  m   = (1.0/3.0)*pow(r,3)*rho_c;

  fp = fopen("density", "w");

  while (r < MAXSTEPS*DR) {

    fprintf(fp, "%le %le\n", r, rho);

    /* Take a RK step: */
    k1m = DR*DmDr(r, rho);
    k1rho = DR*DrhoDr(r, rho, m);

    k2m = DR*DmDr(r + DR/2.0, rho + k1rho/2.0);
    k2rho = DR*DrhoDr(r + DR/2.0, rho + k1rho/2.0, m + k1m/2.0);

    k3m = DR*DmDr(r + DR/2.0, rho + k2rho/2.0);
    k3rho = DR*DrhoDr(r + DR/2.0, rho + k2rho/2.0, m + k2m/2.0);

    k4m = DR*DmDr(r + DR, rho + k3rho);
    k4rho = DR*DrhoDr(r + DR, rho + k3rho, m + k3m);

    m   += (k1m + k4m)/6.0 + (k2m + k3m)/3.0;
    rho += (k1rho + k4rho)/6.0 + (k2rho + k3rho)/3.0;
    r   += DR;

    /* Check if we have reached the edge of the star: */
    if (rho <= 0.0) {
      printf("\n");
      printf("Radius (km):\t%4.3e\n", r*R0/1000.0);
      printf("Mass (M_sun):\t%4.3e\n", m*M0/M_SOLAR);
      fclose (fp);
      exit(0);
    }
  }
  /* If we get here, the edge of the star was not reached... */
  printf("Model failed to converge after %d steps\n", MAXSTEPS);
  printf("Final r   = %le\n", r);
  printf("Final rho = %le\n", rho);
  printf("Final m   = %le\n", m);
  fclose (fp);
  return 1;
}

